/*	AA script rendering an SVG file, with clickable areas corresponding to a Blocks property.

    Copyright (c) 2020 PIXILAB Technologies AB, Sweden (http://pixilab.se). All Rights Reserved.
 */
(function () {
    /**
     * Instantiate a PubSubPeer that auto-connects to the blocks server
     * using a websocket.
     *
     * The PIXILAB Websocket API is defined in the pub-sub-peer.js script, which
     * attaches its root object as a global (window) variable named PIXILAB_BLOCKS.
     */
    var pubSubPeer = new PIXILAB_BLOCKS.PubSubPeer(onServerConnectionChange);
    var blocksVarName = getQueryParams()['blocksVar'];
    var svgFileName = getQueryParams()['svgFileName'];
    /**
     * The server connection state has changed. Here we just log that event, but you
     * may want to use it to indicate the server is temporarily offline, or similar.
     */
    function onServerConnectionChange(connected) {
        console.log(connected ? 'Connected to the Blocks server' : 'Disconnected from the Blocks server');
    }
    /**
     * Get the set of query params specified in URL, building it if not yet done.
     */
    function getQueryParams() {
        if (!queryParams) {
            var query = window.location.search.substring(1);
            var vars = query.split("&");
            var qParams = {};
            if (vars.length > 0 && vars[0].length > 0) { // Got some fish
                for (var i = 0; i < vars.length; i++) {
                    var pair = vars[i].split('=');
                    var value = pair[1];
                    value = value.split('?')[0]; // Discard any garbage added by SSSP v2
                    qParams[pair[0]] = value;
                }
            }
            queryParams = qParams;
        }
        return queryParams;
    }
    var queryParams; // Use ONLY through getQueryParams
    /**
     *	Load an external SVG file into the DOM as a SVG element. Listen for
     *	clicks on children of #TheMap inside the SVG to select clicked
     *	child, indicated by applying the class 'svg-selected' to the selected child.
     */
    var SvgMap = /** @class */ (function () {
        /*	Create me by loading SVG from external file.
         */
        function SvgMap(addToEl) {
            var _this = this;
            this.addToEl = addToEl;
            fetch(svgFileName). // Selected in the query parameter svgFileName.
                then(function (response) {
                if (response.ok)
                    return response.text();
                else
                    throw response.statusText + ': ' + response.url;
            }).
                then(function (svgDoc) { return _this.addSvgDoc(svgDoc); }).
                catch(function (error) { return addToEl.innerText = error; }); // Provides some error ind
            // const dataHandler = {dataReceived: this.onVariableValueChange};
            // this.onVariableValueChange(pubSubPeer.subscribe(blocksVarName,dataHandler ));
        }
        /**	Turn addTo XML data into an SVG node and add inside el.
         */
        SvgMap.prototype.addSvgDoc = function (svgDoc) {
            // Turn string into elements we can work with
            var elems = SvgMap.parseXml(svgDoc);
            // Look in that collection for the svg element in there
            for (var eix = 0; eix < elems.length; ++eix) {
                var svg = elems[eix];
                if (svg.tagName === 'svg') {
                    // Discard attribute not kosher for inline svg
                    svg.removeAttribute('xmlns:a');
                    this.addSvgEl(svg);
                    return;
                }
            }
        };
        /**
         * Attempt to intepret string into something DOM-like, returning an HTMLCollection
         */
        SvgMap.parseXml = function (str) {
            var tmp = document.implementation.createHTMLDocument();
            tmp.body.innerHTML = str;
            return tmp.body.children;
        };
        /**
         * Add the SVG element to the DOM. Look for our "interesting" element group,  attach
         * a click handler there and start to subscribe to the Blocks variable.
         */
        SvgMap.prototype.addSvgEl = function (svg) {
            var _this = this;
            this.addToEl.appendChild(svg);
            var theMap = this.svgWrap = document.getElementById('TheMap');
            if (theMap) {
                theMap.addEventListener('mousedown', function (evt) {
                    _this.clickedEl(_this.findIdentifiedSvgEl(evt.target));
                });
                var dataHandler = { dataReceived: function (blocksData) {
                        return _this.onVariableValueChange(blocksData);
                    }
                };
                this.onVariableValueChange(pubSubPeer.subscribe(blocksVarName, dataHandler));
            }
        };
        /**
         * Starting from el, find the closest element upwards in the hierarchy
         * that has an ID, and return it. Return undefined if none found prior
         * to reaching our wrapper.
         */
        SvgMap.prototype.findIdentifiedSvgEl = function (el) {
            do {
                if (el.id) // Found one with ID
                    return el;
                el = el.parentElement; // Continue up the hierarchy
            } while (el !== this.svgWrap); // Stop once we reach wrapping el
        };
        /**
         * Things to do if an element has been clicked.
         */
        SvgMap.prototype.clickedEl = function (el) {
            pubSubPeer.set(blocksVarName, el ? el.id : ""); //update Blocks variable on new selections
        };
        /**
         * Things to do when variable is updated in Blocks
         */
        SvgMap.prototype.onVariableValueChange = function (blocksData) {
            if (blocksData !== undefined) {
                var el = document.getElementById(blocksData);
                this.selectEl(el);
            }
        };
        /**
         * Select el by applying svg-selected class to it. Remove from any
         * previously selected. If el is undefined, then only remove from
         * any previously selected.
         */
        SvgMap.prototype.selectEl = function (el) {
            if (this.lastSelected && this.lastSelected !== el) {
                this.lastSelected.classList.remove('svg-selected');
                this.lastSelected = undefined;
            }
            if (el) {
                el.classList.add('svg-selected');
                this.lastSelected = el;
            }
            return;
        };
        return SvgMap;
    }());
    // Get us going
    new SvgMap(document.getElementById('svg'));
})();
//# sourceMappingURL=script.js.map